@@ -9,7 +9,7 @@
 namespace Mercator {
 
 template <typename DataType>
-Buffer<DataType>::Buffer(Segment & segment, unsigned int channels) :
+Buffer<DataType>::Buffer(const Segment & segment, unsigned int channels) :
          m_segment(segment), m_channels(channels), m_size(segment.getSize()),
          m_data(0)
 {
