@@ -22,19 +22,31 @@
 
 namespace Mercator {
 
-// A couple of helper classes
-// These interpolate between points or across a quad
 
+/// \brief Helper to interpolate on a line.
+///
+/// The line specified is of integer length, and the position specified
+/// as an integer. A check is included to avoid calculation if the value
+/// at each end is the same.
 class LinInterp {
   private:
+    /// The length of the line.
     int m_size;
+    /// Flag indicating that both points have the same value.
     bool noCalc;
   public:
+    /// Values at the two ends.
     float ep1, ep2;
+    /// \brief Determine the interpolated value along the line.
     inline float calc(int loc) 
     {
         return ((noCalc) ? ep1 : ((m_size-loc) * ep1 + loc * ep2));
     }
+    /// \brief Constructor
+    ///
+    /// @param size length of the line.
+    /// @param l value at one end of the line.
+    /// @param h value at one end of the line.
     LinInterp(int size,float l, float h) : m_size(size), noCalc(false), 
               ep1(l/size), ep2(h/size) 
     {
@@ -45,18 +57,35 @@ class LinInterp {
     } 
 };
 
+/// \brief Helper to interpolate in a quad.
+///
+/// The quad specified is assumed to be square of integer size, and
+/// the position specified for interpolation is specified in integer
+/// form. A check is included to avoid calculation if the value of each
+/// corner is the same.
 class QuadInterp {
   private:
+    /// The length of one side of the square quad.
     int m_size;
+    /// Flag indicating that all points have the same value.
     bool noCalc;
   public:
+    /// Values at the four corners.
     float ep1, ep2, ep3, ep4;
+    /// \brief Determine the interpolated value within the quad.
     inline float calc(int locX, int locY) 
     {
         return  ((noCalc) ? ep1 :
                 (( ep1*(m_size-locX) + ep2 * locX) * (m_size-locY) +
                 ( ep4*(m_size-locX) + ep3 * locX) * (locY) ) / m_size );
     }
+    /// \brief Constructor
+    ///
+    /// @param size length of one side of the square quad.
+    /// @param e1 value at one corner of the square quad.
+    /// @param e2 value at one corner of the square quad.
+    /// @param e3 value at one corner of the square quad.
+    /// @param e4 value at one corner of the square quad.
     QuadInterp(int size,float e1, float e2, float e3, float e4)
         : m_size(size), noCalc(false),
           ep1(e1/size), ep2(e2/size), ep3(e3/size), ep4(e4/size) 
@@ -90,7 +119,7 @@ Segment::Segment(int x, int y, unsigned int resolution) :
 /// Generally it is not necessary to delete Segment objects from application
 /// code, as Segment instances are owned by the Terrain object.
 /// Storage allocated for heightfield and surface normals is implicitly
-/// deleted.
+/// deleted as well as all surfaces.
 Segment::~Segment()
 {
     clearMods();
@@ -100,6 +129,13 @@ Segment::~Segment()
     if (m_normals != 0) {
         delete [] m_normals;
     }
+    
+    Segment::Surfacestore::const_iterator I = m_surfaces.begin();
+    Segment::Surfacestore::const_iterator Iend = m_surfaces.end();
+    for(; I != Iend; ++I) {
+        delete I->second;
+    }
+    
 }
 
 /// \brief Populate the Segment with heightfield data.
@@ -127,9 +163,7 @@ void Segment::populate() // const Matrix<2, 2, BasePoint> & base)
 /// This is called internally whenever changes occur that mean that the
 /// heightfield and surface normal data are no longer valid.
 /// If surface normal storage is deallocated, and if the points argument
-/// is true the heightfield storage is also deallocated. The
-/// Surface::invalidate() method is called for each surface associated with
-/// this Segment.
+/// is true the heightfield storage is also deallocated.
 void Segment::invalidate(bool points)
 {
     if (points && m_points != 0) {
@@ -144,6 +178,11 @@ void Segment::invalidate(bool points)
     invalidateSurfaces();
 }
 
+/// \brief Mark surfaces as stale.
+///
+/// This is called internally from Segment::invalidate() when changes occur
+/// that mean the surface data is no longer valid. The Surface::invalidate()
+/// method is called for each surface.
 void Segment::invalidateSurfaces()
 {
     Segment::Surfacestore::const_iterator I = m_surfaces.begin();
@@ -521,8 +560,8 @@ void Segment::getHeightAndNormal(float x, float y, float& h,
 /// @param ly lower y coordinate of intersection area.
 /// @param hy upper y coordinate of intersection area.
 /// @return true if the box intersects with this Segment, false otherwise.
-bool Segment::clipToSegment(const WFMath::AxisBox<2> &bbox, int &lx, int &hx,
-                                                            int &ly, int &hy) 
+bool Segment::clipToSegment(const WFMath::AxisBox<2> &bbox,
+                            int &lx, int &hx, int &ly, int &hy) const
 {
     lx = I_ROUND(bbox.lowCorner()[0]); 
     if (lx > m_res) return false;
@@ -549,24 +588,28 @@ bool Segment::clipToSegment(const WFMath::AxisBox<2> &bbox, int &lx, int &hx,
 /// the modification will be applied directly.
 void Segment::addMod(TerrainMod *t) 
 {
-    m_modList.push_back(t);
+    m_modList.insert(t);
     if (isValid()) {
         applyMod(t);
     }
 }
 
+/// \brief Remove a TerrainMod from this Segment.
+///
+/// Called from Terrain::removeMod().
+void Segment::removeMod(TerrainMod * tm)
+{
+    m_modList.erase(tm);
+    invalidate();
+}
+
 /// \brief Delete all the modifications applied to this Segment.
 ///
 /// Usually called from the destructor. It is not normally necessary to call
 /// this function from the application.
 void Segment::clearMods() 
 {
-    if (m_modList.size()) {
-        ModList::iterator I = m_modList.begin();
-        ModList::iterator Iend = m_modList.end();
-        for (; I != Iend; ++I) {
-            delete(*I);
-        }
+    if (m_modList.size() != 0) {
         m_modList.clear();
         invalidate();
     }
@@ -594,22 +637,42 @@ void Segment::applyMod(TerrainMod *t)
     invalidate(false);
 }
 
+/// \brief Add an area to those that affect this segment.
+///
+/// Call from Terrain when an Area is added which is found to intersect this
+/// segment.
+/// @param ar the area to be added.
 void Segment::addArea(Area* ar)
 {
     m_areas.insert(Areastore::value_type(ar->getLayer(), ar));
     invalidateSurfaces();
 }
 
+/// \brief Remove an area from those that affect this segment.
+void Segment::removeArea(Area* area)
+{
+    Areastore::iterator I = m_areas.lower_bound(area->getLayer());
+    Areastore::iterator Iend = m_areas.upper_bound(area->getLayer());
+    while (I != Iend) {
+        Areastore::iterator II = I;
+        ++I;
+        if (II->second == area) {
+            m_areas.erase(II);
+        }
+    }
+    invalidateSurfaces();
+}
+
 WFMath::AxisBox<2> Segment::getRect() const
 {
-    WFMath::Point<2> lp(m_xRef * m_res, m_yRef * m_res), 
+    WFMath::Point<2> lp(m_xRef, m_yRef), 
         hp(lp.x() + m_res, lp.y() + m_res);
     return WFMath::AxisBox<2>(lp, hp);
 }
 
 WFMath::AxisBox<3> Segment::getBox() const
 {
-    WFMath::Point<3> lp(m_xRef * m_res, m_yRef * m_res, m_min), 
+    WFMath::Point<3> lp(m_xRef, m_yRef, m_min), 
         hp(lp.x() + m_res, lp.y() + m_res, m_max);
     return WFMath::AxisBox<3>(lp, hp);
 }
