@@ -4,6 +4,13 @@
 
 #include <Mercator/Plant.h>
 
+#ifdef NDEBUG
+#undef NDEBUG
+#endif
+#ifndef DEBUG
+#define DEBUG
+#endif
+
 #include <cassert>
 
 int main()
